@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2017 The Qt Company Ltd.
 ** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the Qt Quick Templates 2 module of the Qt Toolkit.
@@ -52,6 +52,7 @@
 
 QT_BEGIN_NAMESPACE
 
+class QValidator;
 class QQuickPopup;
 class QQmlInstanceModel;
 class QQuickComboBoxPrivate;
@@ -62,8 +63,7 @@ class Q_QUICKTEMPLATES2_PRIVATE_EXPORT QQuickComboBox : public QQuickControl
     Q_PROPERTY(int count READ count NOTIFY countChanged FINAL)
     Q_PROPERTY(QVariant model READ model WRITE setModel NOTIFY modelChanged FINAL)
     Q_PROPERTY(QQmlInstanceModel *delegateModel READ delegateModel NOTIFY delegateModelChanged FINAL)
-    Q_PROPERTY(bool flat READ isFlat WRITE setFlat NOTIFY flatChanged FINAL REVISION 1)
-    Q_PROPERTY(bool pressed READ isPressed WRITE setPressed NOTIFY pressedChanged FINAL)
+    Q_PROPERTY(bool pressed READ isPressed WRITE setPressed NOTIFY pressedChanged FINAL) // ### Qt 6: should not be writable
     Q_PROPERTY(int highlightedIndex READ highlightedIndex NOTIFY highlightedIndexChanged FINAL)
     Q_PROPERTY(int currentIndex READ currentIndex WRITE setCurrentIndex NOTIFY currentIndexChanged FINAL)
     Q_PROPERTY(QString currentText READ currentText NOTIFY currentTextChanged FINAL)
@@ -72,6 +72,23 @@ class Q_QUICKTEMPLATES2_PRIVATE_EXPORT QQuickComboBox : public QQuickControl
     Q_PROPERTY(QQmlComponent *delegate READ delegate WRITE setDelegate NOTIFY delegateChanged FINAL)
     Q_PROPERTY(QQuickItem *indicator READ indicator WRITE setIndicator NOTIFY indicatorChanged FINAL)
     Q_PROPERTY(QQuickPopup *popup READ popup WRITE setPopup NOTIFY popupChanged FINAL)
+    // 2.1 (Qt 5.8)
+    Q_PROPERTY(bool flat READ isFlat WRITE setFlat NOTIFY flatChanged FINAL REVISION 1)
+    // 2.2 (Qt 5.9)
+    Q_PROPERTY(bool down READ isDown WRITE setDown RESET resetDown NOTIFY downChanged FINAL REVISION 2)
+    Q_PROPERTY(bool editable READ isEditable WRITE setEditable NOTIFY editableChanged FINAL REVISION 2)
+    Q_PROPERTY(QString editText READ editText WRITE setEditText RESET resetEditText NOTIFY editTextChanged FINAL REVISION 2)
+    Q_PROPERTY(QValidator *validator READ validator WRITE setValidator NOTIFY validatorChanged FINAL REVISION 2)
+    Q_PROPERTY(Qt::InputMethodHints inputMethodHints READ inputMethodHints WRITE setInputMethodHints NOTIFY inputMethodHintsChanged FINAL REVISION 2)
+    Q_PROPERTY(bool inputMethodComposing READ isInputMethodComposing NOTIFY inputMethodComposingChanged FINAL REVISION 2)
+    Q_PROPERTY(bool acceptableInput READ hasAcceptableInput NOTIFY acceptableInputChanged FINAL REVISION 2)
+    // 2.5 (Qt 5.12)
+    Q_PROPERTY(qreal implicitIndicatorWidth READ implicitIndicatorWidth NOTIFY implicitIndicatorWidthChanged FINAL REVISION 5)
+    Q_PROPERTY(qreal implicitIndicatorHeight READ implicitIndicatorHeight NOTIFY implicitIndicatorHeightChanged FINAL REVISION 5)
+    Q_CLASSINFO("DeferredPropertyNames", "background,contentItem,indicator,popup")
+    // 2.14 (Qt 5.14)
+    Q_PROPERTY(QVariant currentValue READ currentValue NOTIFY currentValueChanged FINAL REVISION 14)
+    Q_PROPERTY(QString valueRole READ valueRole WRITE setValueRole NOTIFY valueRoleChanged FINAL REVISION 14)
 
 public:
     explicit QQuickComboBox(QQuickItem *parent = nullptr);
@@ -83,9 +100,6 @@ public:
     void setModel(const QVariant &model);
     QQmlInstanceModel *delegateModel() const;
 
-    bool isFlat() const;
-    void setFlat(bool flat);
-
     bool isPressed() const;
     void setPressed(bool pressed);
 
@@ -103,6 +117,9 @@ public:
     QString textRole() const;
     void setTextRole(const QString &role);
 
+    QString valueRole() const;
+    void setValueRole(const QString &role);
+
     QQmlComponent *delegate() const;
     void setDelegate(QQmlComponent *delegate);
 
@@ -115,15 +132,51 @@ public:
     Q_INVOKABLE QString textAt(int index) const;
     Q_INVOKABLE int find(const QString &text, Qt::MatchFlags flags = Qt::MatchExactly) const;
 
+    // 2.1 (Qt 5.8)
+    bool isFlat() const;
+    void setFlat(bool flat);
+
+    // 2.2 (Qt 5.9)
+    bool isDown() const;
+    void setDown(bool down);
+    void resetDown();
+
+    bool isEditable() const;
+    void setEditable(bool editable);
+
+    QString editText() const;
+    void setEditText(const QString &text);
+    void resetEditText();
+
+    QValidator *validator() const;
+    void setValidator(QValidator *validator);
+
+    Qt::InputMethodHints inputMethodHints() const;
+    void setInputMethodHints(Qt::InputMethodHints hints);
+
+    bool isInputMethodComposing() const;
+    bool hasAcceptableInput() const;
+
+    // 2.5 (Qt 5.12)
+    qreal implicitIndicatorWidth() const;
+    qreal implicitIndicatorHeight() const;
+
+    // 2.14 (Qt 5.14)
+    QVariant currentValue() const;
+    Q_REVISION(14) Q_INVOKABLE QVariant valueAt(int index) const;
+    Q_REVISION(14) Q_INVOKABLE int indexOfValue(const QVariant &value) const;
+
 public Q_SLOTS:
     void incrementCurrentIndex();
     void decrementCurrentIndex();
+    Q_REVISION(2) void selectAll();
 
 Q_SIGNALS:
+    void activated(int index);
+    void highlighted(int index);
     void countChanged();
     void modelChanged();
     void delegateModelChanged();
-    Q_REVISION(1) void flatChanged();
     void pressedChanged();
     void highlightedIndexChanged();
     void currentIndexChanged();
@@ -133,25 +186,46 @@ Q_SIGNALS:
     void delegateChanged();
     void indicatorChanged();
     void popupChanged();
-
-    void activated(int index);
-    void highlighted(int index);
+    // 2.1 (Qt 5.8)
+    Q_REVISION(1) void flatChanged();
+    // 2.2 (Qt 5.9)
+    Q_REVISION(2) void accepted();
+    Q_REVISION(2) void downChanged();
+    Q_REVISION(2) void editableChanged();
+    Q_REVISION(2) void editTextChanged();
+    Q_REVISION(2) void validatorChanged();
+    Q_REVISION(2) void inputMethodHintsChanged();
+    Q_REVISION(2) void inputMethodComposingChanged();
+    Q_REVISION(2) void acceptableInputChanged();
+    // 2.5 (Qt 5.12)
+    Q_REVISION(5) void implicitIndicatorWidthChanged();
+    Q_REVISION(5) void implicitIndicatorHeightChanged();
+    // 2.14 (Qt 5.14)
+    Q_REVISION(14) void valueRoleChanged();
+    Q_REVISION(14) void currentValueChanged();
 
 protected:
+    bool eventFilter(QObject *object, QEvent *event) override;
+    void focusInEvent(QFocusEvent *event) override;
     void focusOutEvent(QFocusEvent *event) override;
+#if QT_CONFIG(im)
+    void inputMethodEvent(QInputMethodEvent *event) override;
+#endif
     void keyPressEvent(QKeyEvent *event) override;
     void keyReleaseEvent(QKeyEvent *event) override;
-    void mousePressEvent(QMouseEvent *event) override;
-    void mouseMoveEvent(QMouseEvent *event) override;
-    void mouseReleaseEvent(QMouseEvent *event) override;
-    void mouseUngrabEvent() override;
+#if QT_CONFIG(wheelevent)
     void wheelEvent(QWheelEvent *event) override;
+#endif
 
     void componentComplete() override;
+    void itemChange(ItemChange change, const ItemChangeData &value) override;
+    void contentItemChange(QQuickItem *newItem, QQuickItem *oldItem) override;
+    void localeChange(const QLocale &newLocale, const QLocale &oldLocale) override;
 
     QFont defaultFont() const override;
+    QPalette defaultPalette() const override;
 
-#ifndef QT_NO_ACCESSIBILITY
+#if QT_CONFIG(accessibility)
     QAccessible::Role accessibleRole() const override;
     void accessibilityActiveChanged(bool active) override;
 #endif
